﻿#------------------------------------------------------------------------------
# Name:        zielerreichungsgrad_arcgis.py
# Version:     1.0
#
# Purpose:     Berechnung Zielerreichungsgrad Oekomorphologie
#
# Author:      Sigmaplan AG, Bern
#
# Created:     November 2016
# Copyright:   (c) BAFU
#
# Python:      2.7.5
# ArcGIS:      10.1 (ArcView License)
#------------------------------------------------------------------------------

import os
import arcpy
import datetime
import wertefunktion

starttime = datetime.datetime.now()

# Benutzereingaben und Umgebung

in_gdb = arcpy.GetParameterAsText(0) + "\\"
out_gdb = arcpy.GetParameterAsText(1) + "\\"

print "input: ", in_gdb
print "output: ", out_gdb

# Weitere Variabeln und Umgebung
arcpy.env.workspace = out_gdb
arcpy.env.overwriteOutput = True

def msgOut(msg):
    print msg
    arcpy.AddMessage(msg)

def errOut(err):
    print err
    print arcpy.GetMessages()
    print datetime.datetime.now()-starttime
    arcpy.AddError(err)
    arcpy.AddError(arcpy.GetMessages())
    arcpy.AddError(datetime.datetime.now()-starttime)


# Routen-Events ueberlagern
try:
    msgOut(u"Superposition des évènements d’itinéraires......")
    e_b01 = "B01"
    e_b02 = "B02"
    e_c01 = "C01"
    e_c02 = "C02"
    e_c03 = "C03"
    e_c04 = "C04"
    e_c05 = "C05"
    e_c06 = "C06"
    e_d01 = "D01"
    e_d02 = "D02"
    e_e01 = "E01"
    e_e02 = "E02"

    props = "ID_Uferlinie LINE Von Bis"
    arcpy.OverlayRouteEvents_lr(in_gdb + e_b01, props, in_gdb + e_b02, props, "UNION", out_gdb + "Overlay_02", props, "NO_ZERO")
    arcpy.OverlayRouteEvents_lr(out_gdb + "Overlay_02", props, in_gdb + e_c01, props, "UNION", out_gdb + "Overlay_03", props, "NO_ZERO")
    arcpy.OverlayRouteEvents_lr(out_gdb + "Overlay_03", props, in_gdb + e_c02, props, "UNION", out_gdb + "Overlay_04", props, "NO_ZERO")
    arcpy.OverlayRouteEvents_lr(out_gdb + "Overlay_04", props, in_gdb + e_c03, props, "UNION", out_gdb + "Overlay_05", props, "NO_ZERO")
    arcpy.OverlayRouteEvents_lr(out_gdb + "Overlay_05", props, in_gdb + e_c04, props, "UNION", out_gdb + "Overlay_06", props, "NO_ZERO")
    arcpy.OverlayRouteEvents_lr(out_gdb + "Overlay_06", props, in_gdb + e_c05, props, "UNION", out_gdb + "Overlay_07", props, "NO_ZERO")
    arcpy.OverlayRouteEvents_lr(out_gdb + "Overlay_07", props, in_gdb + e_c06, props, "UNION", out_gdb + "Overlay_08", props, "NO_ZERO")
    arcpy.OverlayRouteEvents_lr(out_gdb + "Overlay_08", props, in_gdb + e_d01, props, "UNION", out_gdb + "Overlay_09", props, "NO_ZERO")
    arcpy.OverlayRouteEvents_lr(out_gdb + "Overlay_09", props, in_gdb + e_d02, props, "UNION", out_gdb + "Overlay_10", props, "NO_ZERO")
    arcpy.OverlayRouteEvents_lr(out_gdb + "Overlay_10", props, in_gdb + e_e01, props, "UNION", out_gdb + "Overlay_11", props, "NO_ZERO")
    arcpy.OverlayRouteEvents_lr(out_gdb + "Overlay_11", props, in_gdb + e_e02, props, "UNION", out_gdb + "Overlay_all", props, "NO_ZERO")

    msgOut(u"Modification des noms de champs...")

    fields = ["B01", "B02", "C01", "C02", "C03", "C04", "C05", "C06", "D01",
              "D02", "E01", "E02"]
    for field in fields:
        dump_field = arcpy.ListFields(out_gdb + "Overlay_all", "Code*")[0]
        arcpy.AddField_management(out_gdb + "Overlay_all", field, "TEXT", field_length = 50)
        arcpy.CalculateField_management(out_gdb + "Overlay_all", field, "!" + dump_field.name + "!", "PYTHON_9.3")
        arcpy.DeleteField_management(out_gdb + "Overlay_all", dump_field.name)

    arcpy.DeleteField_management(out_gdb + "Overlay_all", ["Bemerkungen",
                                                           "Bemerkungen2",
                                                           "Bemerkungen3",
                                                           "Bemerkungen4",
                                                           "Bemerkungen5",
                                                           "Bemerkungen6",
                                                           "Bemerkungen7",
                                                           "Bemerkungen8",
                                                           "Bemerkungen9",
                                                           "Bemerkungen10",
                                                           "Bemerkungen11",
                                                           "Bemerkungen12"
                                                           ])
    msgOut(u"Suppression des tables temporaires...")

    delete_tbl = [tbl for tbl in arcpy.ListTables() if not tbl.endswith('_all') and tbl.startswith('Overlay')]
    for dtbl in delete_tbl:
        arcpy.Delete_management(dtbl)
except:
    errOut(u"ERREUR : une erreur est survenue lors de la superposition des évènements d’itinéraire")

# Dictionary mit Bewertung der Codes
code_dict = {
            "B01.01":"1.0", "B01.02":"0.75", "B01.03":"0.5", "B01.04":"0.25",
            "B01.05":"0.0", "B01.06":"0.0", "B02.01":"1.0", "B02.02":"1.0",
            "B02.03":"0.25", "B02.04":"0.0", "C01.01":"1.0", "C01.02":"0.75",
            "C01.03":"0.5", "C01.04":"0.5", "C01.05":"0.25", "C01.06":"0.0",
            "C01.07":"0.0", "C02.01":"1.0", "C02.02":"0.75", "C02.03":"0.5",
            "C02.04":"0.25", "C02.05":"0.0", "C03.01":"1.0", "C03.02":"0.75",
            "C03.03":"0.5", "C03.04":"0.5", "C03.05":"0.25", "C03.06":"0.25",
            "C03.07":"0.0", "C03.08":"0.0", "C03.09":"0.0", "C03.10":"0.0",
            "C03.11":"0.0", "C04.01":"1.0", "C04.02":"0.75", "C04.03":"0.25",
            "C04.04":"0.5", "C05.01":"1.0", "C05.02":"1.0", "C05.03":"0.5",
            "C05.04":"0.25", "C05.05":"0.0", "C05.06":"0.0", "C05.07":"0.0",
            "C06.01":"1.0", "C06.02":"1.0", "C06.03":"0.0", "C06.04":"0.0",
            "D01.01":"1.0", "D01.02":"0.75", "D01.03":"0.5", "D01.04":"0.25",
            "D01.05":"0.0", "D02.01":"1.0", "D02.02":"1.0", "D02.03":"0.0",
            "D02.04":"0.0", "E01.01":"1.0", "E01.02":"0.75", "E01.03":"0.75",
            "E01.04":"0.5", "E01.05":"0.5", "E01.06":"0.25", "E01.07":"0.25",
            "E01.08":"0.0", "E02.01":"1.0", "E02.02":"0.75", "E02.03":"0.5",
            "E02.04":"0.25", "E02.05":"0.0",
            }

# Codes in Bewertung uebersetzen
try:
    msgOut(u"Conversion des codes d’attribut en évaluations...")
    for field in fields:
        arcpy.RemoveDomainFromField_management(out_gdb + "Overlay_all", field)
    for field in fields:
        with arcpy.da.UpdateCursor(out_gdb + "Overlay_all", field) as rows:
            for row in rows:
                print row[0]
                if row[0] == None or row[0] == '': # <Null>-Values, z.B. aufgrund von Erfassungsluecken
                    row[0] = 99
                    rows.updateRow(row)
                elif row[0][:6] in code_dict:
                    x = row[0][:6]
                    row[0] = code_dict.get(x) # Umweg via Variable notwendig
                    rows.updateRow(row)
                else: # Alle weiteren ungueltigen Werte, z.B. wenn Domains umgangen wurden
                    row[0] = 99
                    rows.updateRow(row)
except:
    errOut(u"ERREUR : une erreur est survenue lors de la conversion des codes d’attribut en évaluations")

# Zieltabelle erstellen
try:
    msgOut(u"Création de la table cible « Zielerreichungsgrade »...")

    arcpy.CreateTable_management(out_gdb, "Zielerreichungsgrade")

    # ID-Feld abhaengig vom Input-Datentyp anfuegen
    id_type = arcpy.ListFields(out_gdb + "Overlay_all", "ID_Uferlinie")[0].type
    id_length = arcpy.ListFields(out_gdb + "Overlay_all", "ID_Uferlinie")[0].length
    msgOut(u"ID type: " + id_type)

    if id_type == "String":
        arcpy.AddField_management(out_gdb + "Zielerreichungsgrade", "ID_Uferlinie", id_type, "", "", id_length)
    else:
        arcpy.AddField_management(out_gdb + "Zielerreichungsgrade", "ID_Uferlinie", id_type)

    # Weitere, Input-unabhaengige Felder anfuegen
    arcpy.AddField_management(out_gdb + "Zielerreichungsgrade", "NN_Oekomorphologie", "DOUBLE")
    arcpy.AddField_management(out_gdb + "Zielerreichungsgrade", "NN_Uferlinie", "DOUBLE")
    arcpy.AddField_management(out_gdb + "Zielerreichungsgrade", "NN_Uferzone", "DOUBLE")
    arcpy.AddField_management(out_gdb + "Zielerreichungsgrade", "NN_Uferstreifen", "DOUBLE")
    arcpy.AddField_management(out_gdb + "Zielerreichungsgrade", "NN_FlachWZ", "DOUBLE")
    arcpy.AddField_management(out_gdb + "Zielerreichungsgrade", "NN_Hinterlandstr", "DOUBLE")
    arcpy.AddField_management(out_gdb + "Zielerreichungsgrade", "KeineNutzung_US", "DOUBLE")
    arcpy.AddField_management(out_gdb + "Zielerreichungsgrade", "Von", "DOUBLE")
    arcpy.AddField_management(out_gdb + "Zielerreichungsgrade", "Bis", "DOUBLE")
except:
    errOut(u"ERREUR : une erreur est survenue lors de la création de la table cible « Zielerreichungsgrade »...")

# Zielhierarchie berechnen und schreiben
try:
    msgOut(u"Calcul et écriture de la hiérarchie des objectifs...")
    search_fields = ["ID_Uferlinie", "Von", "Bis"] + fields
    insert_fields = ["ID_Uferlinie", "Von", "Bis", "KeineNutzung_US",
                     "NN_Uferstreifen", "NN_Hinterlandstr", "NN_Uferzone",
                     "NN_Uferlinie", "NN_FlachWZ", "NN_Oekomorphologie"
                    ]

    with arcpy.da.SearchCursor(out_gdb + "Overlay_all", search_fields) as srows:
        for srow in srows:
            msgOut(srow)
            values = wertefunktion.zielhierarchie(float(srow[3]), float(srow[4]), float(srow[5]),
                                                  float(srow[6]), float(srow[7]), float(srow[8]),
                                                  float(srow[9]), float(srow[10]), float(srow[11]),
                                                  float(srow[12]), float(srow[13]), float(srow[14]))
            msgOut(values)

            with arcpy.da.InsertCursor(out_gdb + "Zielerreichungsgrade", insert_fields) as irows:
                irows.insertRow([srows[0], srows[1], srows[2], values[0],
                                 values[1], values[2], values[3],
                                 values[4], values[5], values[6]])
                msgOut("inserted row")
except:
    errOut(u"ERREUR : une erreur est survenue lors du calcul et de l’écriture de la hiérarchie des objectifs")

# Temoraere Tabelle loeschen
try:
    msgOut(u"Suppression des tables temporaires...")
    arcpy.Delete_management(out_gdb + "Overlay_all")
except:
    errOut(u"ERREUR : une erreur est survenue lors de la suppression des tables temporaires")

msgOut(u"Exécution du script « Zielerreichungsgrade » terminée")
msgOut("Durée: " + str(datetime.datetime.now()-starttime))
