﻿#------------------------------------------------------------------------------
# Name:        wertefunktion.py
# Version:     1.0
#
# Purpose:     Wertefunktion der Berechnung Zielerreichungsgrad Oekomorphologie
#
# Author:      Sigmaplan AG, Bern
#
# Created:     November 2016
# Copyright:   (c) BAFU
#
# Python:      2.7.5
#------------------------------------------------------------------------------

import os
import math
import datetime

starttime = datetime.datetime.now()

# Gewichtungen Bonus/Malus
bonus_nn_ufersaum = 0.3
bonus_nn_uebergang = 0.3
malus_nn_hinterland = 0.5
malus_nn_flachwasser = 0.7

# Aggregatfunktionen

def minimum(*v):
    """Bestimmt den tiefsten Wert der untergeordnenten Zielhierarchieebene und
    gibt diesen zurueck
    """
    try:
        if min(v) >= 0 and min(v) <= 1:
            v = min(v)
        else:
            v = 99
        return v
    except:
        print "ERROR: Fehler in Berechnung mit Aggregatfunktion 'Minimum'"
        print "Dauer: ", datetime.datetime.now()-starttime

def bonus(vv, vb, wb):
    """Wertet den Zielerreichungsgrad auf, falls das Bonusziel besser ist als
    das Vorrangziel und gibt den entsprechenden Wert zurueck
    """
    try:
        if  vv >= vb:
            v = vv
        elif vv < vb:
            v = wb*vb+(1-wb)*vv
        else:
            v = 99
        return v
    except:
        print "ERROR: Fehler in Berechnung mit Aggregatfunktion 'Bonus'"
        print "Dauer: ", datetime.datetime.now()-starttime

def malus(vv, vm, wm):
    """Wertet den Zielerreichungsgrad ab, falls das Malusziel schlechter ist
    als das Vorrangziel und gibt den entsprechenden Wert zurueck
    """
    try:
        if vv <= vm:
            v = vv
        elif vv > vm:
            v = wm*vm+(1-wm)*vv
        else:
            v = 99
        return v
    except:
        print "ERROR: Fehler in Berechnung mit Aggregatfunktion 'malus'"
        print "Dauer: ", datetime.datetime.now()-starttime

def transform_additiv(v1, v2, vm, wm):
    """Wertet den Zielerreichungsgrad abhaengig der untergeordneten Ziel-
    erreichungsgrade mehr oder weniger auf und gibt den entsprechenden
    Wert zurueck
    """
    try:
        if v1 <= 0.5:
            g_v1 = 0.5*math.pow(2*v1, 2)
        elif v1 <= 1:
            g_v1 = 1-0.5*math.pow(2*(1-v1), 2)
        else:
            g_v1 = 99
        if v2 <= 0.5:
            g_v2 = 0.5*math.pow(2*v2, 2)
        elif v2 <= 1:
            g_v2 = 1-0.5*math.pow(2*(1-v2), 2)
        else:
            g_v2 = 99
        g = 0.5*(g_v1+g_v2)
        if g <= 0.5:
            vv = 0.5*math.sqrt(2*g)
        elif g <= 1:
            vv = 1-0.5*math.sqrt(2*(1-g))
        else:
            vv = 99
        if vv <= vm:
            v = vv
        elif vv <= 1:
            v = wm*vm+(1-wm)*vv
        else:
            v = 99
        return v
    except:
        print "ERROR: Fehler in Berechnung mit Aggregatfunktion 'Transformiert-Additiv'"
        print "Dauer: ", datetime.datetime.now()-starttime



# Zielhierarchie berechnen
def zielhierarchie(b01, b02, c01, c02, c03, c04, c05, c06, d01, d02, e01, e02):
    """Berechnet die Zielhierarchie anhand der definierte Aggregationsverfahren
    und gibt die sieben Werte in der Reihenfolge derer Berechnung als Liste
    zurueck
    """
    try:
        if round(minimum(c01, c02, c03, c04, c05), 3) > 1:
            keine_nutzung_us = 99
        else:
            keine_nutzung_us = round(minimum(c01, c02, c03, c04, c05), 3)

        if round(bonus(keine_nutzung_us, c06, bonus_nn_ufersaum), 3) >1:
            nn_uferstreifen = 99
        else:
            nn_uferstreifen = round(bonus(keine_nutzung_us, c06, bonus_nn_ufersaum), 3)

        if round(bonus(d01, d02, bonus_nn_uebergang), 3) >1:
            nn_hinterland = 99
        else:
            nn_hinterland = round(bonus(d01, d02, bonus_nn_uebergang), 3)

        if round(malus(nn_uferstreifen, nn_hinterland, malus_nn_hinterland), 3) >1:
            nn_uferzone = 99
        else:
            nn_uferzone = round(malus(nn_uferstreifen, nn_hinterland, malus_nn_hinterland), 3)

        if round(minimum(b02, b01), 3) >1:
            nn_uferlinie = 99
        else:
            nn_uferlinie = round(minimum(b02, b01), 3)

        if round(minimum(e01, e02), 3) >1:
            nn_flachwasser = 99
        else:
            nn_flachwasser = round(minimum(e01, e02), 3)

        if round(transform_additiv(nn_uferlinie, nn_uferzone, nn_flachwasser, malus_nn_flachwasser), 3) >1:
                nn_oekomorph = 99
        else:
           nn_oekomorph = round(transform_additiv(nn_uferlinie, nn_uferzone,
                             nn_flachwasser, malus_nn_flachwasser), 3)

        ziele = (
                keine_nutzung_us,
                nn_uferstreifen,
                nn_hinterland,
                nn_uferzone,
                nn_uferlinie,
                nn_flachwasser,
                nn_oekomorph
                )

        return ziele
    except:
        print "ERROR: Fehler in Berechnung der Zielhierarchie"
        print "Dauer: ", datetime.datetime.now()-starttime

##print "Dauer: ", datetime.datetime.now()-starttime